@@ -1,6 +1,8 @@
 from airy.core.conf import settings
 from airy.core.exceptions import Http404
 from airy.core.files.uploadedfile import SimpleUploadedFile
+from airy.core.reportbug import report_on_fail
+from airy.utils.encoding import smart_unicode
 from tornado.web import *
 from tornado.escape import *
 from tornadio2 import TornadioRouter, SocketConnection, event
@@ -122,7 +124,7 @@ class AiryRequestHandler(RequestHandler):
 
     def render_string(self, template_name, **kwargs):
         context_processors = getattr(settings, 'template_context_processors', [])
-        template_args = {'reverse_url': self.reverse_url}
+        template_args = {'reverse_url': self.reverse_url, "_utf8": utf8}
         for processor_path in context_processors:
             path, name = processor_path.rsplit('.', 1)
             processor_module = __import__(path, fromlist=path)
@@ -132,10 +134,24 @@ class AiryRequestHandler(RequestHandler):
         return super(AiryRequestHandler, self).render_string(template_name, **template_args)
 
     def is_robot(self, *args, **kwargs):
-        robots = ['Googlebot', ]
+        robots = [
+            'facebook',
+            'postrank',
+            'voyager',
+            'twitterbot',
+            'googlebot',
+            'slurp',
+            'butterfly',
+            'pycurl',
+            'tweetmemebot',
+            'metauri',
+            'evrinid',
+            'reddit',
+            'digg',
+        ]
         robots.extend(args)
         for robot in robots:
-            if robot in self.request.headers['User-Agent']:
+            if robot.lower() in self.request.headers['User-Agent'].lower():
                 return True
         return False
 
@@ -443,7 +459,7 @@ class AiryHandler(object):
 
 
         """
-        return self.execute('airy.request("get", "%s");' % url)
+        return self.execute('airy.ui.redirect("%s");' % url)
 
     def insert(self, target, data):
         """
@@ -477,6 +493,12 @@ class AiryHandler(object):
         """
         return self.execute('airy.ui.remove("%s")' % target)
 
+    def set_title(self, text):
+        """
+        Set page title to ``text``.
+        """
+        return self.execute('airy.ui.title("%s")' % text)
+
     def render_string(self, template_name, **kwargs):
         """
         Render the template specified in ``template_name``.
@@ -507,7 +529,7 @@ class AiryHandler(object):
 
         """
         context_processors = getattr(settings, 'template_context_processors', [])
-        template_args = {'reverse_url': self.reverse_url}
+        template_args = {'reverse_url': self.reverse_url,  "_utf8": utf8}
         for processor_path in context_processors:
             path, name = processor_path.rsplit('.', 1)
             processor_module = __import__(path, fromlist=path)
@@ -558,8 +580,10 @@ class AiryCoreHandler(SocketConnection):
         logging.info('Socket Connected: %s %s' % (self.info.ip, self.info.arguments.get('t', '')))
         site.connections.add(self)
 
+    @event('set_state')
     def set_state(self, state):
-        self.state = state
+        (scheme, host, path, arguments) = self.parse_url(state)
+        self.state = path
 
     def parse_url(self, url):
         "Returns a tuple in the form (scheme, host, path, arguments)"
@@ -568,10 +592,10 @@ class AiryCoreHandler(SocketConnection):
         return parsed.scheme, parsed.netloc, parsed.path, arguments
 
     @event('get')
+    @report_on_fail
     def get(self, url, *args, **kwargs):
         "Main entry point for WebSocket requests"
         (scheme, host, path, arguments) = self.parse_url(url)
-        self.set_state(path)
         handler, hargs, hkwargs = site.resolve_url(path, self, arguments=arguments)
         hargs.extend(args)
         hkwargs.update(kwargs)
@@ -580,7 +604,7 @@ class AiryCoreHandler(SocketConnection):
                                                       self.info.arguments.get('t', ''),
                                                       handler.__class__.__name__))
             try:
-                handler.get(*hargs, **hkwargs)
+                return handler.get(*hargs, **hkwargs)
             except Http404:
                 self.process_404(*hargs, **hkwargs)
         else:
@@ -589,6 +613,7 @@ class AiryCoreHandler(SocketConnection):
             self.process_404(*hargs, **hkwargs)
 
     @event('post')
+    @report_on_fail
     def post(self, url, *args, **kwargs):
         "Main entry point for WebSocket requests"
         (scheme, host, path, arguments) = self.parse_url(url)
@@ -601,7 +626,6 @@ class AiryCoreHandler(SocketConnection):
         except:
             data = {}
         arguments.update(data)
-        self.set_state(path)
         handler, hargs, hkwargs = site.resolve_url(path, self, arguments=arguments)
         hargs.extend(args)
         hkwargs.update(kwargs)
@@ -610,7 +634,7 @@ class AiryCoreHandler(SocketConnection):
                                                       self.info.arguments.get('t', ''),
                                                       handler.__class__.__name__))
             try:
-                handler.post(*hargs, **hkwargs)
+                return handler.post(*hargs, **hkwargs)
             except Http404:
                 self.process_404(*hargs, **hkwargs)
         else:
@@ -632,6 +656,10 @@ class AiryCoreHandler(SocketConnection):
         except AttributeError:
             logging.error('Page with url %s doesn\'t exist' % (kwargs.get('url', ''), ))
 
+    def reverse_url(self, name, *args):
+        """Alias for `Application.reverse_url`."""
+        return self.application.reverse_url(name, *args)
+
     #
     # Airy client interaction
     #
@@ -641,7 +669,7 @@ class AiryCoreHandler(SocketConnection):
         return self
 
     def redirect(self, url):
-        return self.execute('airy.request("get", "%s");' % url)
+        return self.execute('airy.ui.redirect("%s");' % url)
 
     def insert(self, target, data):
         "Insert data into target"
@@ -654,6 +682,12 @@ class AiryCoreHandler(SocketConnection):
     def prepend(self, target, data):
         return self.execute('airy.ui.prepend("%s", %s);' % (target, json_encode(data)))
 
+    def remove(self, target):
+        return self.execute('airy.ui.remove("%s")' % target)
+
+    def set_title(self, text):
+        return self.execute('airy.ui.title("%s");' % text)
+
     def render_string(self, template_name, **kwargs):
         "Render the given template"
         context_processors = getattr(settings, 'template_context_processors', [])
@@ -673,5 +707,18 @@ class AiryCoreHandler(SocketConnection):
         self.insert(target, html)
         return self
 
+def utf8(value):
+    """Converts a string argument to a byte string.
+
+    If the argument is already a byte string or None, it is returned unchanged.
+    Otherwise it must be a unicode string and is encoded as utf8.
+    """
+    if isinstance(value, _UTF8_TYPES):
+        return value
+    assert isinstance(value, unicode)
+    return smart_unicode(value, encoding="utf-8", errors="ignore")
+
+_UTF8_TYPES = (bytes, type(None))
+
 
 core_router = TornadioRouter(AiryCoreHandler)
